-----------------------------------------------------------------------------

local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentGroupMembers")
L:RegisterTranslations("enUS", function() return {
	["Group Members"] = true,
	["Current Group Members"] = true,
	["Show Group Members"] = true,
	["Show Non-Group Members"] = true,
    ["Class"] = true,

	["Automatic Group Reset"] = true,
	["Reset fight data automatically when joining a new group."] = true,
} end)

-----------------------------------------------------------------------------

local profile

-----------------------------------------------------------------------------

Assessment.TagData.Class =
{
	method = function(data)
				local fightData = data.fightData
				if fightData then
					return fightData.groupMembers[data.key]
				end
			 end,
	data = "groupMembers",
	events = "RosterLib_RosterUpdated",
}

Assessment.TagData.IsGroupMember =
{
	method = function(data)
				local fightData = data.fightData
				if fightData then
					return fightData.groupMembers[data.key]
				end
			 end,
	data = "groupMembers",
	events = "RosterLib_RosterUpdated",
}

-----------------------------------------------------------------------------

Assessment.ListType.GroupMembers = 
{
	method = function(fightData) return pairs(fightData.groupMembers) end,
	data = "groupMembers",
	description = L["Group Members"],
}

local groupMemberNameList = { }
Assessment.ListType.CurrentGroupMembers = 
{
	method = function(fightData)
				for k in pairs(groupMemberNameList) do
					groupMemberNameList[k] = nil
				end
				
				for unit in Roster:IterateRoster(true) do
					groupMemberNameList[unit.name] = true
				end
				
				return pairs(groupMemberNameList)
			 end,
	data = "groupMembers",
	events = "RosterLib_RosterUpdated",
	description = L["Current Group Members"],
}

-----------------------------------------------------------------------------

Assessment.FilterType.GroupMembers =
{
	tag			= "IsGroupMember",
	description = L["Show Group Members"]
}

-----------------------------------------------------------------------------

Assessment.FilterType.NonGroupMembers =
{
	tag			= "~IsGroupMember",
	description = L["Show Non-Group Members"]
}

-----------------------------------------------------------------------------

Assessment.ColorType.Class = 
{
	method = function(fightData, data)
				local class = fightData.groupMembers[data.key]
				if not class then return nil end
				return profile.classColors[class]
			 end,
	data = "groupMembers",
	description = L["Class"]
}

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.automaticGroupReset =
{
	type = "toggle",
	name = L["Automatic Group Reset"],
	desc = L["Reset fight data automatically when joining a new group."],
	get = function() return profile.resetOnJoinGroup end,
	set = function(v) profile.resetOnJoinGroup = v end,
}

-----------------------------------------------------------------------------

local AssessmentGroupMembers = Assessment:NewModule("GroupMembers", "AceEvent-2.0")
AssessmentGroupMembers.db = Assessment:AcquireDBNamespace("GroupMembers")
local profileDefaults =
{
	classColors =
	{
	},
	resetOnJoinGroup = true,
}

for k,v in pairs(RAID_CLASS_COLORS) do
	profileDefaults.classColors[k] = { v.r, v.g, v.b }
end
Assessment:RegisterDefaults("GroupMembers", "profile", profileDefaults)
Assessment:RegisterDefaults("GroupMembers", "char", { inGroup = false })

function AssessmentGroupMembers:OnInitialize()
	profile = self.db.profile
	for data in self:ForceIterateUpdateDataTables() do
		if not data.groupMembers then
			self:OnNewFightDataTable(data)
		end
	end
end

function AssessmentGroupMembers:OnEnable()
	self:RegisterEvent("RosterLib_RosterUpdated", "OnRosterUpdated")
	self:RegisterEvent("Assessment:NewFightDataTable", "OnNewFightDataTable")
end

function AssessmentGroupMembers:OnRosterUpdated()
	if profile.resetOnJoinGroup then
		if GetNumPartyMembers() + GetNumRaidMembers() > 0 then
			if not self.db.char.inGroup then
				self.db.char.inGroup = true
				Assessment:ResetUnnamedFightData()
			end
		else
			if self.db.char.inGroup then
				self.db.char.inGroup = false
			end
		end
	end

	for data in self:ForceIterateUpdateDataTables() do
		data.dataUpdateTimes.groupMembers = GetTime()

		local groupMembers = data.groupMembers
		for unit in Roster:IterateRoster(true) do
			groupMembers[unit.name] = unit.class
		end
	end
end

function AssessmentGroupMembers:OnNewFightDataTable(data)
	data.dataUpdateTimes.groupMembers = GetTime()
	
	local groupMembers = self:NewTable()
	data.groupMembers = groupMembers

	for unit in Roster:IterateRoster(true) do
		groupMembers[unit.name] = unit.class
	end
end

-----------------------------------------------------------------------------
